@@ -57,6 +57,11 @@
 #include "usplash.h"
 #include "usplash-theme.h"
 
+/*
+ * This shouldn't be needed any more now that animation takes place
+ * outside the alarm handler.  However, removing it seems to introduce
+ * an Intel driver X initialization problem (LP: #327230).  ?!
+ */
 sigset_t sigs;
 #define blocksig() do{ sigprocmask(SIG_BLOCK, &sigs, NULL); } while(0)
 #define unblocksig() do{ sigprocmask(SIG_UNBLOCK, &sigs, NULL); } while(0)
@@ -122,7 +127,7 @@ static struct {
 	int width;
 	int height;
 	int charcount;
-	char *data;
+	unsigned char *data;
 } saved_font = { 0, 0, 0, NULL };
 
 /* Number of seconds to wait for a command before exiting */
@@ -135,6 +140,11 @@ static int verbose_default = 0;
 /* /dev/console */
 static int console_fd = -1;
 
+/* pipe to trigger animation steps */
+static int alarm_fds[2];
+int animation_needed_fd;
+static int default_pulsate_mode = 0;
+
 struct usplash_funcs {
 	int (*usplash_setfont) (void *font);
 	int (*usplash_getfontwidth) (char c);
@@ -231,6 +241,10 @@ void usplash_text(int x, int y, const char *s, int len, int fg, int bg)
 void usplash_done()
 {
 	usplash_operations->usplash_done();
+
+	close(alarm_fds[1]);
+	close(alarm_fds[0]);
+	animation_needed_fd = alarm_fds[0] = alarm_fds[1] = -1;
 }
 
 void usplash_getdimensions(int *x, int *y)
@@ -379,7 +393,7 @@ int set_font(int width, int height, int count, unsigned char *buf)
 	/* if the count is not 256 or 512, round up and try again */
 	if (errno == EINVAL && width == 8 && count != 256 && count < 512) {
 		int ct = ((count > 256) ? 512 : 256);
-		char *mybuf = malloc(32 * ct);
+		unsigned char *mybuf = malloc(32 * ct);
 		int ret;
 
 		if (!mybuf) {
@@ -436,6 +450,16 @@ void usplash_restore_font(void)
 		 saved_font.charcount, saved_font.data);
 }
 
+int usplash_theme_width(struct usplash_theme* theme)
+{
+	return theme->theme_width ? theme->theme_width : theme->pixmap->width;
+}
+
+int usplash_theme_height(struct usplash_theme* theme)
+{
+	return theme->theme_height ? theme->theme_height : theme->pixmap->height;
+}
+
 int usplash_setup(int xres, int yres, int v)
 {
 	int ret;
@@ -465,7 +489,6 @@ int usplash_setup(int xres, int yres, int v)
 	if (xres == 0 || yres == 0) {
 		FILE *conf;
 		char line[1024];
-		int res;
 		conf = fopen("/etc/usplash.conf", "r");
 		if (conf) {
 			while (!feof(conf) && 
@@ -476,8 +499,8 @@ int usplash_setup(int xres, int yres, int v)
 		} else {
 		  /* Pick the first one, which by convention is
 			   the lowest resolution one. */
-			xres = theme->pixmap->width; 
-			yres = theme->pixmap->height; 
+			xres = usplash_theme_width(theme);
+			yres = usplash_theme_height(theme);
 		}
 	}
 
@@ -498,15 +521,13 @@ int usplash_setup(int xres, int yres, int v)
 	ratio =
 	    (float) xres / (float) yres >
 	    1.55 ? USPLASH_16_9 : USPLASH_4_3;
+
 	while (theme) {
-		if (theme->pixmap->height <= yres
-		    && theme->pixmap->width <= xres
-		    && theme->pixmap->height *
-		    theme->pixmap->width > maxarea
+		if (usplash_theme_height(theme) <= yres
+		    && usplash_theme_width(theme) <= xres
+		    && usplash_theme_height(theme) * usplash_theme_width(theme) > maxarea
 		    && theme->ratio == ratio) {
-			maxarea =
-			    theme->pixmap->height *
-			    theme->pixmap->width;
+			maxarea = usplash_theme_height(theme) * usplash_theme_width(theme);
 			htheme = theme;
 		}
 		theme = theme->next;
@@ -523,8 +544,8 @@ int usplash_setup(int xres, int yres, int v)
 	if (theme->init)
 		theme->init(theme);
 
-	left_edge = (usplash_xres - theme->pixmap->width) / 2;
-	top_edge = (usplash_yres - theme->pixmap->height) / 2;
+	left_edge = (usplash_xres - usplash_theme_width(theme)) / 2;
+	top_edge = (usplash_yres - usplash_theme_height(theme)) / 2;
 	text_x1 = left_edge + theme->text_x;
 	text_y1 = top_edge + theme->text_y;
 	text_x2 = text_x1 + theme->text_width;
@@ -536,6 +557,13 @@ int usplash_setup(int xres, int yres, int v)
 
 	usplash_set_palette(theme->pixmap->ncols, theme->pixmap->palette);
 
+	alarm_fds[0] = alarm_fds[1] = -1;
+	pipe(alarm_fds);
+	animation_needed_fd = alarm_fds[0];
+	if (fcntl(animation_needed_fd, F_SETFL, O_NONBLOCK)<0) {
+		perror("fcntl");
+	}
+
 	sigemptyset(&sigs);
 	sigaddset(&sigs, SIGALRM);
 	return 0;
@@ -560,7 +588,8 @@ void clear_screen(void)
 	else {
 		usplash_clear(0, 0, usplash_xres, usplash_yres,
 			      theme->background);
-		usplash_put(left_edge, top_edge, theme->pixmap);
+		usplash_put(left_edge + theme->pixmap_x, top_edge +
+			theme->pixmap_y, theme->pixmap);
 	}
 	unblocksig();
 }
@@ -733,11 +762,11 @@ void flush_stdin()
 char get_timeout_char(int inputtimeout)
 {
 	char charinput = -1;
-	int old_flags,retval;
-	struct pollfd input_fd = {
-		.fd = STDIN_FILENO,
-		.events = POLLIN,
-	};
+	struct pollfd fds[2];
+	fds[0].fd = STDIN_FILENO;
+	fds[0].events = POLLIN;
+	fds[1].fd = animation_needed_fd;
+	fds[1].events = POLLIN;
         struct termios t;
 
         /* configure unbuffered input */
@@ -749,15 +778,18 @@ char get_timeout_char(int inputtimeout)
 
 	while( inputtimeout )
 	{
-		blocksig();
-		if( poll( &input_fd, 1 , 100) > 0 )
+		if( poll( fds, 2 , 100) > 0 )
 		{
-			if( read(STDIN_FILENO,&charinput,1) < 0 )
-				charinput = '\0';
-			inputtimeout = 1; // like "break", but unblocksig-safe
+			if ( fds[0].revents & POLLIN ) {
+				if( read(STDIN_FILENO,&charinput,1) < 0 )
+					charinput = '\0';
+				inputtimeout = 1; // like "break", but unblocksig-safe
+			}
+			if ( fds[1].revents & POLLIN ) {
+				process_pending_animation_steps();
+			}
 		}
-		unblocksig();
-		inputtimeout--;
+		if (inputtimeout > 0) inputtimeout--;
 	}
 	return charinput;
 }
@@ -770,13 +802,9 @@ int usplash_timeout_get_string(char *inputbuf, int length, int quiet,int inputti
 	
 	/* Get user input */
 	for (i = 0; i < length - 1; i++) {
-		if( inputtimeout == -1 )
-			input = getchar();
-		else{
-			input = get_timeout_char(inputtimeout);
-                        if ( input == -1 )
-                                break;
-                }
+		input = get_timeout_char(inputtimeout);
+		if ( input == -1 )
+			break;
 		if (input == '\n' || input == '\r' || input == '\0')
 			break;
 
@@ -851,25 +879,23 @@ static int _open_out_fifo()
 /* handle input with a timeout in seconds. timeout = -1 means no timeout */
 int handle_timeout_input(const char *string, const size_t len, const int quiet,int inputtimeout)
 {
-	int i;
 	ssize_t wlen;
 	int fifo_outfd = -1;
 	int reset_verbose = 0;
 	char inputbuf[PIPE_BUF];
 	int err = 0;
 
-	/* Initialize text area if not running verbose yet */
+	/* need to be verbose for string output */
 	if (!verbose) {
 		reset_verbose = 1;
 		verbose = 1;
-		clear_text();
 	}
 
 	/* draw the prompt */
 	draw_text(string, len);
 
 	/* Get the user input */
-	usplash_timeout_get_string(inputbuf, PIPE_BUF, quiet,inputtimeout);
+	usplash_timeout_get_string(inputbuf, PIPE_BUF, quiet, inputtimeout);
 
 	/* Reset the verbose flag */
 	if (reset_verbose)
@@ -899,7 +925,6 @@ int handle_input_char()
 {
         char ch;
 	int fifo_outfd = -1;
-        int i;
 	int err = 0;
 
         ch = get_timeout_char(1);
@@ -935,7 +960,7 @@ int handle_verbose (int mode)
             verbose = 0;
             break;
         default:
-            fprintf(stderr, "usplash: invalid verbosity mode %si\n", mode);
+            fprintf(stderr, "usplash: invalid verbosity mode %i\n", mode);
             err = 1;
     }
 
@@ -971,11 +996,56 @@ void draw_status(const char *string, size_t len, int mode)
 	unblocksig();
 }
 
-void animate_step(pulsating)
+void set_pulsating(int pulsate_enabled)
+{
+	default_pulsate_mode = pulsate_enabled;
+}
+
+void request_animate_step()
+{
+	static char dummy = '\0';
+	write(alarm_fds[1], &dummy, 1);
+}
+
+/* catch up on any pending animation steps */
+void process_pending_animation_steps()
+{
+	char dummy;
+	int bytes;
+	for (;;) {
+		bytes = read(animation_needed_fd, &dummy, 1);
+		if (bytes == 1) {
+			 animate_step(-1);
+		}
+		else if (bytes < 0) {
+			if (errno == EINTR) continue;
+			/* Treat expected EAGAIN and unexpected other errno
+			   values the same way: stop the animation cycle. */
+			return;
+		}
+		else {
+			/* Something unexpected has happened: EOF, large read */
+			return;
+		}
+	}
+}
+
+/*
+  pulsating =  1  throbber should be drawn
+  pulsating =  0  throbber should not be drawn
+  pulsating = -1  fall back to pulsate value set via set_pulsating()
+*/
+void animate_step(int pulsating)
 {
 	static int pulsate_step = 0;
 	static int num_steps = 37;
 	int x1, y1, x2, y2;
+
+	/* fall back to externally defined pulsate mode */
+	if (pulsating < 0) {
+		pulsating = default_pulsate_mode;
+	}
+
 	if (theme->animate_step)
 		theme->animate_step(theme, pulsating);
 	else {
