@@ -30,6 +30,9 @@ void draw_text(const char *string, size_t len);
 void draw_text_urgent(const char *string, size_t len);
 void draw_line(const char *string, size_t len);
 void draw_status(const char *string, size_t len, int mode);
+void set_pulsating(int pulsate_enabled);
+void request_animate_step();
+void process_pending_animation_steps();
 void animate_step(int pulsating);
 int usplash_setup(int xres, int yres, int verbose);
 int strncspn(const char *s, size_t n, const char *reject);
@@ -44,3 +47,4 @@ extern struct usplash_theme testcard_theme;
 extern struct usplash_theme *theme;
 extern int usplash_xres, usplash_yres;
 extern int top_edge, left_edge;
+extern int animation_needed_fd;
