@@ -30,6 +30,11 @@
 #include <signal.h>
 #include <stdint.h>
 
+/*
+ * This shouldn't be needed any more now that animation takes place
+ * outside the alarm handler.  However, removing it seems to introduce
+ * an Intel driver X initialization problem (LP: #327230).  ?!
+ */
 extern sigset_t sigs;
 #define blocksig() do{ sigprocmask(SIG_BLOCK, &sigs, NULL); } while(0)
 
@@ -134,7 +139,6 @@ int usplash_svga_setmode() {
 int usplash_svga_init()
 {
 	int rc = 0;
-	int mode;
 
 	vga_setchipset(VESA);
 
@@ -198,11 +202,12 @@ void usplash_svga_text(int x, int y, const char *s, int len, int fg,
 			c = myfont->content[index + i];
 			/* Plot */
 			for (j = 0; j < width; j++) {
-				if (c & (1 << (32 - j)))
+				if (c & (1 << (32 - j))) {
                                         if (bpp == 8)
                                                 gl_setpixel(x + j, y + i, fg);
                                         else
                                                 gl_setpixelrgb(x + j, y + i, vesa_palette[fg].red, vesa_palette[fg].green, vesa_palette[fg].blue);
+				}
 			}
 		}
 		x += width;
@@ -242,8 +247,8 @@ void usplash_svga_put_part(int x, int y, int width, int height,
 	int i;
 	int size = width * height;
 	struct usplash_pixmap *pixmap = pointer;
-	unsigned char *part = (char *) malloc(size * sizeof(char));
-	char *start = pixmap->data + y0 * pixmap->width + x0;
+	unsigned char *part = malloc(size * sizeof(unsigned char));
+	unsigned char *start = pixmap->data + y0 * pixmap->width + x0;
 	for (i = 0; i < height; i++) {
 		memcpy(part + i * width, start + i * pixmap->width, width);
 	}
